/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     codegen.c                                               *
 * Created:       2001-05-28 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-06-03 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#include "hasm64.h"


static int cg_set_size (codegen_t *cg, unsigned long n);


codegen_t *cg_new (unsigned long n)
{
  codegen_t *cg;

  cg = (codegen_t *) malloc (sizeof (codegen_t));
  if (cg == NULL) {
    return (NULL);
  }

  cg->addr = 0;
  cg->buf_i1 = 0;
  cg->buf_i2 = 0;
  cg->buf_n = 0;
  cg->buf = NULL;

  cg_set_size (cg, n);

  return (cg);
}

void cg_del (codegen_t *cg)
{
  if (cg != NULL) {
    if (cg->buf != NULL) {
      free (cg->buf);
    }

    free (cg);
  }
}

void cg_clear (codegen_t *cg)
{
  cg->buf_i1 = 0;
  cg->buf_i2 = 0;
}

static
int cg_set_size (codegen_t *cg, unsigned long n)
{
  unsigned      i;
  unsigned long buf_n;
  unsigned char *buf;

  if (n <= cg->buf_n) {
    return (0);
  }

  if (n > 65536UL) {
    return (1);
  }

  buf_n = cg->buf_n;
  if (buf_n == 0) {
    buf_n = 1;
  }

  while (buf_n < n) {
    buf_n *= 2;
  }

  if (buf_n > 65536UL) {
    buf_n = 65536UL;
  }

  buf = (unsigned char *) malloc (buf_n);
  if (buf == NULL) {
    return (1);
  }

  for (i = 0; i < cg->buf_i1; i++) {
    buf[i] = cg->buf[i];
  }

  for (i = cg->buf_i1; i < cg->buf_i2; i++) {
    buf[i] = 0;
  }

  free (cg->buf);

  cg->buf = buf;

  return (0);
}

void cg_set_addr (codegen_t *cg, unsigned addr)
{
  cg->addr = addr;
}

unsigned cg_get_addr (codegen_t *cg)
{
  return (cg->addr);
}

unsigned cg_get_index (codegen_t *cg)
{
  return (cg->buf_i2);
}

int cg_add_byte (codegen_t *cg, unsigned char val, unsigned cnt)
{
  unsigned long i, n;

  n = (unsigned long)cg->buf_i2 + cnt;
  if (n > 65536UL) {
    return (1);
  }

  if (cg_set_size (cg, n)) {
    return (1);
  }

  for (i = cg->buf_i2; i < n; i++) {
    cg->buf[i] = val;
  }

  cg->buf_i1 = n;
  cg->buf_i2 = n;
  cg->addr = (cg->addr + cnt) & 0xffff;

  return (0);
}

int cg_res_byte (codegen_t *cg, unsigned cnt)
{
  unsigned long n;

  n = (unsigned long)cg->buf_i2 + cnt;
  if (n > 65536UL) {
    return (1);
  }

  cg->buf_i2 = n;

  return (0);
}

int cg_rep_bytes (codegen_t *cg, unsigned i1, unsigned i2, unsigned cnt)
{
  unsigned      i, j;
  unsigned long n, a;

  n = (unsigned long)cnt * (i2 - i1 + 1);
  a = cg->buf_i2 + n;
  if (a > 65536UL) {
    return (1);
  }

  if (cg_set_size (cg, a)) {
    return (1);
  }

  j = cg->buf_i2;

  while (cnt > 0) {
    for (i = i1; i <= i2; i++) {
      cg->buf[j] = cg->buf[i];
      j += 1;
    }

    cnt -= 1;
  }

  cg->buf_i1 = a;
  cg->buf_i2 = a;
  cg->addr = (cg->addr + n) & 0xffff;

  return (0);
}

int cg_write (codegen_t *cg, FILE *fp)
{
  fwrite (cg->buf, 1, cg->buf_i1, fp);

  return (0);
}

int cg_write_file (codegen_t *cg, char *fname)
{
  int  r;
  FILE *fp;

  fp = fopen (fname, "wb");
  if (fp == NULL) {
    return (1);
  }

  r = cg_write (cg, fp);

  fclose (fp);

  return (r);
}
