/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     codegen.h                                               *
 * Created:       2001-05-28 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-06-03 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#ifndef HASM64_CODEGEN_H
#define HASM64_CODEGEN_H


typedef struct {
  unsigned      addr;
  unsigned      buf_i1;
  unsigned      buf_i2;
  unsigned long buf_n;
  unsigned char *buf;
} codegen_t;


/*!************************************************************************
 * Create a new code generator with the initial buffer size n.
 **************************************************************************/
codegen_t *cg_new (unsigned long n);

/*!************************************************************************
 * Delete a code generator.
 **************************************************************************/
void cg_del (codegen_t *cg);

/*!************************************************************************
 * Clear the output buffer.
 **************************************************************************/
void cg_clear (codegen_t *cg);

/*!************************************************************************
 * Set the current address. This does not affect the output buffer.
 **************************************************************************/
void cg_set_addr (codegen_t *cg, unsigned offs);

/*!************************************************************************
 * Get the current address.
 **************************************************************************/
unsigned cg_get_addr (codegen_t *cg);

/*!************************************************************************
 * Get the current index. This is the number of bytes written to the
 * output buffer so far.
 **************************************************************************/
unsigned cg_get_index (codegen_t *cg);

/*!************************************************************************
 * Add a byte cnt times to the output buffer.
 **************************************************************************/
int cg_add_byte (codegen_t *cg, unsigned char val, unsigned cnt);

/*!************************************************************************
 * Reserve cnt bytes in the output buffer.
 **************************************************************************/
int cg_res_byte (codegen_t *cg, unsigned cnt);

int cg_rep_bytes (codegen_t *cg, unsigned i1, unsigned i2, unsigned cnt);

/*!************************************************************************
 * Write the output buffer to a file.
 **************************************************************************/
int cg_write (codegen_t *cg, FILE *fp);

/*!************************************************************************
 * Write the output buffer to a file.
 **************************************************************************/
int cg_write_file (codegen_t *cg, char *fname);


#endif
