/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     expr.c                                                  *
 * Created:       2001-05-30 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-06-03 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#include "hasm64.h"


sumxy_t *par_exp_xy (parser_t *par);


int par_is_number (parser_t *par)
{
  if (par->scn->tok_id == TOK_DIGIT) {
    return (1);
  }

  return (0);
}

int par_number (parser_t *par, unsigned *val)
{
  unsigned i, n;
  unsigned base, dig;
  char     *str;

  str = par_str (par);

  n = strlen (str);

  if (str[0] == '$') {
    base = 16;
    i = 1;
  }
  else if ((str[0] == '0') && (str[1] == 'x')) {
    base = 16;
    i = 2;
  }
  else if ((str[0] == '0') && (str[1] == 'b')) {
    base = 2;
    i = 2;
  }
  else {
    base = 10;
    i = 0;
  }

  *val = 0;

  while (i < n) {
    if ((str[i] >= '0') && (str[i] <= '9')) {
      dig = (unsigned)(str[i] - '0');
    }
    else if ((str[i] >= 'a') && (str[i] <= 'f')) {
      dig = (unsigned)(str[i] - 'a' + 10);
    }
    else if ((str[i] >= 'A') && (str[i] <= 'F')) {
      dig = (unsigned)(str[i] - 'A' + 10);
    }
    else {
      return (1);
    }

    if (dig >= base) {
      err_print (par, ERR_ANY, "Illegal digit");
      return (1);
    }

    *val = base * *val + dig;

    i += 1;
  }

  *val &= 0xffff;

  par_scan (par);

  return (0);
}

int par_exp_val (parser_t *par, unsigned *val)
{
  char     str[PAR_MAX_LABEL];
  symbol_t *sym;
  unsigned id;

  id = par_id (par);

  if (id == TOK_SYM_LPAR) {
    par_scan (par);

    if (par_exp (par, val)) {
      return (1);
    }

    if (par_id (par) != TOK_SYM_RPAR) {
      err_print (par, ERR_SYNTAX, "')' expected");
      return (1);
    }

    par_scan (par);

    return (0);
  }
  else if (par_is_number (par)) {
    return (par_number (par, val));
  }
  else if (id == TOK_IDENT) {
    sym = st_sym_get (par->st, par_str (par));

    if (sym == NULL) {
      if (par->pass == 0) {
        *val = 0;
        par->newpass = 1;
        par_scan (par);

        return (0);
      }
      else {
        err_print (par, ERR_ANY, "Undefined symbol");
        return (1);
      }
    }

    *val = sym_get_val (sym);

    par_scan (par);

    return (0);
  }
  else if (id == TOK_SYM_PERIOD) {
    par_scan (par);

    if (par_id (par) != TOK_IDENT) {
      err_print (par, ERR_ANY, "Local label expected");
      return (1);
    }

    par_get_local_label (par, str, par_str (par));

    sym = st_sym_get (par->st, str);

    if (sym == NULL) {
      if (par->pass == 0) {
        *val = 0;
        par->newpass = 1;
        par_scan (par);

        return (0);
      }
      else {
        err_print (par, ERR_ANY, "Undefined symbol");
        return (1);
      }
    }

    *val = sym_get_val (sym);
    par_scan (par);

    return (0);
  }
  else if ((id == TOK_SYM_LO) || (id == TOK_SYM_HI)) {
    par_scan (par);

    if (par_id (par) != TOK_SYM_LPAR) {
      err_print (par, ERR_SYNTAX, "'(' expected");
      return (1);
    }

    par_scan (par);

    if (par_exp (par, val)) {
      return (1);
    }

    if (par_id (par) != TOK_SYM_RPAR) {
      err_print (par, ERR_SYNTAX, "')' expected");
      return (1);
    }

    if (id == TOK_SYM_LO) {
      *val = *val & 0xff;
    }
    else {
      *val = (*val >> 8) & 0xff;
    }

    par_scan (par);

    return (0);
  }
  else if (id == TOK_SYM_PC) {
    par_scan (par);

    *val = cg_get_addr (par->cg);

    return (0);
  }

  err_print (par, ERR_ANY, "Value expected");

  return (1);
}

int par_is_sign (parser_t *par)
{
  switch (par_id (par)) {
    case TOK_SYM_PLUS:
    case TOK_SYM_MINUS:
    case TOK_SYM_NOT:
      return (1);

    default:
      return (0);
  }

  return (0);
}

int par_exp_sign (parser_t *par, unsigned *val)
{
  unsigned op;

  op = par_id (par);

  if (par_is_sign (par)) {
    par_scan (par);
  }

  if (par_exp_val (par, val)) {
    return (1);
  }

  switch (op) {
    case TOK_SYM_PLUS:
      break;

    case TOK_SYM_MINUS:
      *val = (~*val + 1) & 0xffff;
      break;

    case TOK_SYM_NOT:
      *val = (~*val) & 0xffff;
      break;

    default:
      break;
  }

  return (0);
}

int par_is_mul (parser_t *par)
{
  if (par_id (par) < TOK_SYM_MUL_START) {
    return (0);
  }

  if (par_id (par) > TOK_SYM_MUL_END) {
    return (0);
  }

  return (1);
}

int par_exp_mul (parser_t *par, unsigned *val)
{
  unsigned op;
  unsigned r;

  if (par_exp_sign (par, val)) {
    return (1);
  }

  op = par_id (par);

  while (par_is_mul (par)) {
    par_scan (par);

    if (par_exp_sign (par, &r)) {
      return (1);
    }

    switch (op) {
      case TOK_SYM_MUL:
        *val = (*val * r) & 0xffff;
        break;

      case TOK_SYM_DIV:
        if (r == 0) {
          err_print (par, ERR_ANY, "Division by zero");
          return (1);
        }
        else {
          *val = (*val / r);
        }
        break;

      case TOK_SYM_MOD:
        if (r == 0) {
          err_print (par, ERR_ANY, "Division by zero");
          return (1);
        }
        else {
          *val = (*val % r);
        }
        break;

      default:
        err_print (par, ERR_ANY, "Internal error (mul)");
        return (1);
        break;
    }

    op = par_id (par);
  }

  return (0);
}

int par_is_shift (parser_t *par)
{
  if (par_id (par) < TOK_SYM_SHIFT_START) {
    return (0);
  }

  if (par_id (par) > TOK_SYM_SHIFT_END) {
    return (0);
  }

  return (1);
}

int par_exp_shift (parser_t *par, unsigned *val)
{
  unsigned op;
  unsigned r;

  if (par_exp_mul (par, val)) {
    return (1);
  }

  op = par_id (par);

  while (par_is_shift (par)) {
    par_scan (par);

    if (par_exp_mul (par, &r)) {
      return (1);
    }

    switch (op) {
      case TOK_SYM_SHL:
        if (r >= 16) {
          *val = 0;
        }
        else {
          *val = (*val << r) & 0xffff;
        }
        break;

      case TOK_SYM_SHR:
        if (r >= 16) {
          *val = 0;
        }
        else {
          *val = (*val >> r) & 0xffff;
        }
        break;

      default:
        err_print (par, ERR_ANY, "Internal error (shift)");
        return (1);
    }

    op = par_id (par);
  }

  return (0);
}

int par_is_sum (parser_t *par)
{
  if (par_id (par) < TOK_SYM_SUM_START) {
    return (0);
  }

  if (par_id (par) > TOK_SYM_SUM_END) {
    return (0);
  }

  return (1);
}

int par_exp_sum (parser_t *par, unsigned *val)
{
  unsigned op;
  unsigned r;

  if (par_exp_shift (par, val)) {
    return (1);
  }

  op = par_id (par);

  while (par_is_sum (par)) {
    par_scan (par);

    if (par_exp_shift (par, &r)) {
      return (1);
    }

    switch (op) {
      case TOK_SYM_PLUS:
        *val = (*val + r) & 0xffff;
        break;

      case TOK_SYM_MINUS:
        *val = (*val - r) & 0xffff;
        break;

      default:
        err_print (par, ERR_ANY, "Internal error (sum)");
        return (1);
    }

    op = par_id (par);
  }

  return (0);
}

int par_is_bit (parser_t *par)
{
  if (par_id (par) < TOK_SYM_BIT_START) {
    return (0);
  }

  if (par_id (par) > TOK_SYM_BIT_END) {
    return (0);
  }

  return (1);
}

int par_exp_bit (parser_t *par, unsigned *val)
{
  unsigned op;
  unsigned r;

  if (par_exp_sum (par, val)) {
    return (1);
  }

  op = par_id (par);

  while (par_is_bit (par)) {
    par_scan (par);

    if (par_exp_sum (par, &r)) {
      return (1);
    }

    switch (op) {
      case TOK_SYM_OR:
        *val = (*val | r);
        break;

      case TOK_SYM_AND:
        *val = (*val & r);
        break;

      case TOK_SYM_XOR:
        *val = (*val ^ r);
        break;

      default:
        err_print (par, ERR_INTERNAL, "Internal error (bit)");
        return (1);
    }

    op = par_id (par);
  }

  return (0);
}

int par_is_rel (parser_t *par)
{
  if (par_id (par) < TOK_SYM_REL_START) {
    return (0);
  }

  if (par_id (par) > TOK_SYM_REL_END) {
    return (0);
  }

  return (1);
}

int par_exp_rel (parser_t *par, unsigned *val)
{
  unsigned op;
  unsigned r;

  if (par_exp_bit (par, val)) {
    return (1);
  }

  op = par_id (par);

  while (par_is_rel (par)) {
    par_scan (par);

    if (par_exp_bit (par, &r)) {
      return (1);
    }

    switch (op) {
      case TOK_SYM_EQU:
        *val = (*val == r);
        break;

      case TOK_SYM_NEQU:
        *val = (*val != r);
        break;

      case TOK_SYM_LESS:
        *val = (*val < r);
        break;

      case TOK_SYM_GRTR:
        *val = (*val > r);
        break;

      case TOK_SYM_LEQU:
        *val = (*val <= r);
        break;

      case TOK_SYM_GEQU:
        *val = (*val >= r);
        break;

      default:
        err_print (par, ERR_INTERNAL, "Internal error (rel)");
        return (1);
    }

    op = par_id (par);
  }

  return (0);
}

int par_exp (parser_t *par, unsigned *val)
{
  return (par_exp_rel (par, val));
}

sumxy_t *sumxy_new (void)
{
  sumxy_t *ret;

  ret = (sumxy_t *) malloc (sizeof (sumxy_t));
  if (ret == NULL) {
    return (NULL);
  }

  ret->ind = NULL;
  ret->val = 0;
  ret->x = 0;
  ret->y = 0;

  return (ret);
}

void sumxy_del (sumxy_t *sum)
{
  if (sum != NULL) {
    sumxy_del (sum->ind);
    free (sum);
  }
}

int sumxy_check (sumxy_t *sum)
{
  if ((sum->x > 1) || (sum->y > 1)) {
    return (1);
  }

  if ((sum->x != 0) && (sum->y != 0)) {
    return (1);
  }

  if (sum->ind != NULL) {
    if ((sum->val != 0) || (sum->x != 0)) {
      return (1);
    }

    return (sumxy_check (sum->ind));
  }

  return (0);
}

sumxy_t *par_exp_xy_shift (parser_t *par)
{
  sumxy_t *sum;

  sum = sumxy_new ();

  switch (par_id (par)) {
    case TOK_SYM_X:
      par_scan (par);
      sum->x = 1;
      break;

    case TOK_SYM_Y:
      par_scan (par);
      sum->y = 1;
      break;

    case TOK_SYM_LSPAR:
      par_scan (par);

      sum->ind = par_exp_xy_sum (par);
      if (sum->ind == NULL) {
        sumxy_del (sum);
        return (NULL);
      }

      if (par_id (par) != TOK_SYM_RSPAR) {
        sumxy_del (sum);
        err_print (par, ERR_SYNTAX, "']' expected");
        return (NULL);
      }

      par_scan (par);
      break;

    default:
      if (par_exp_shift (par, &sum->val)) {
        sumxy_del (sum);
        return (NULL);
      }
      break;
  }

  return (sum);
}

sumxy_t *par_exp_xy_sum (parser_t *par)
{
  unsigned op;
  sumxy_t  *val1, *val2;

  val1 = par_exp_xy_shift (par);
  if (val1 == NULL) {
    return (NULL);
  }

  op = par_id (par);

  while (par_is_sum (par)) {
    par_scan (par);

    val2 = par_exp_xy_shift (par);
    if (val2 == NULL) {
      sumxy_del (val1);
      return (NULL);
    }

    switch (op) {
      case TOK_SYM_PLUS:
        if (val1->ind == NULL) {
          val1->ind = val2->ind;
          val2->ind = NULL;
        }
        else if (val2->ind != NULL) {
          sumxy_del (val1);
          sumxy_del (val2);
          err_print (par, ERR_ANY, "Illegal indirection");
          return (NULL);
        }

        val1->val += val2->val;
        val1->x += val2->x;
        val1->y += val2->y;

        sumxy_del (val2);

        break;

      case TOK_SYM_MINUS:
        if (val2->ind != NULL) {
          sumxy_del (val1);
          sumxy_del (val2);
          err_print (par, ERR_ANY, "Illegal indirection");
          return (NULL);
        }

        val1->val -= val2->val;
        val1->x -= val2->x;
        val1->y -= val2->y;

        sumxy_del (val2);

        break;

      default:
        sumxy_del (val1);
        sumxy_del (val2);
        err_print (par, ERR_INTERNAL, "Internal error (sumxy)");
        return (NULL);
    }

    op = par_id (par);
  }

  val1->val &= 0xffff;
  val1->x &= 0xffff;
  val1->y &= 0xffff;

  return (val1);
}

sumxy_t *par_exp_xy (parser_t *par)
{
  sumxy_t *val;

  if (par_id (par) != TOK_SYM_LSPAR) {
    if (par_id (par) == TOK_SYM_IMM) {
      par_scan (par);
    }

    val = sumxy_new ();

    if (par_exp (par, &val->val)) {
      sumxy_del (val);
      return (NULL);
    }

    return (val);
  }

  return (par_exp_xy_sum (par));
}
