/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     parser.h                                                *
 * Created:       1997-09-04 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-06-03 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#ifndef HASM64_PARSER_H
#define HASM64_PARSER_H


#define EA_IND_X 0
#define EA_ZPG   1
#define EA_IMM   2
#define EA_ABS   3
#define EA_IND_Y 4
#define EA_ZPG_X 5
#define EA_ABS_Y 6
#define EA_ABS_X 7
#define EA_A     8

#define M_IND_X (1 << EA_IND_X)
#define M_ZPG   (1 << EA_ZPG)
#define M_IMM   (1 << EA_IMM)
#define M_DIR   (1 << EA_ABS)
#define M_IND_Y (1 << EA_IND_Y)
#define M_ZPG_X (1 << EA_ZPG_X)
#define M_DIR_Y (1 << EA_ABS_Y)
#define M_DIR_X (1 << EA_ABS_X)
#define M_A     (1 << EA_A)

#define M_ALL   255


#define COND_PL 0
#define COND_MI 1
#define COND_VC 2
#define COND_VS 3
#define COND_CC 4
#define COND_CS 5
#define COND_NE 6
#define COND_EQ 7

#define PAR_MAX_LABEL 256


/*!************************************************************************
 * The parser type
 **************************************************************************/
typedef struct {
  scanner_t *scn;
  symtab_t  *st;
  codegen_t *cg;

  unsigned  pass;
  int       newpass;

  char      label[PAR_MAX_LABEL];

  char      *mod_name;
  unsigned  mod_base;
} parser_t;


/*!************************************************************************
 * Create a new parser.
 **************************************************************************/
parser_t *par_new (void);

/*!************************************************************************
 * Delete a parser.
 **************************************************************************/
void par_del (parser_t *par);

/*!************************************************************************
 * Set the scanner to be used by the parser.
 **************************************************************************/
void par_set_scn (parser_t *par, scanner_t *scn);

/*!************************************************************************
 * Set the current global label.
 **************************************************************************/
void par_set_label (parser_t *par, char *str);

/*!************************************************************************
 * Get the current global label.
 **************************************************************************/
char *par_get_label (parser_t *par);

/*!************************************************************************
 * Get the current scan token string.
 **************************************************************************/
char *par_str (parser_t *par);

/*!************************************************************************
 * Get the current scan token id.
 **************************************************************************/
unsigned par_id (parser_t *par);

/*!************************************************************************
 * Scan the next token.
 **************************************************************************/
void par_scan (parser_t *par);

/*!************************************************************************
 * Set the base address of the current module.
 **************************************************************************/
void par_set_base (parser_t *par, unsigned base);

/*!************************************************************************
 * Get the current base address.
 **************************************************************************/
unsigned par_get_base (parser_t *par);

/*!************************************************************************
 * Convert the local label src to a global label using the current
 * global label as a prefix.
 **************************************************************************/
int par_get_local_label (parser_t *par, char *dst, char *src);

/*!************************************************************************
 * Parse a module.
 **************************************************************************/
int par_module (parser_t *par);


#endif
