/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     symtab.c                                                *
 * Created:       1997-09-04 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-05-31 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#include "hasm64.h"


symbol_t *sym_new (char *name, unsigned val)
{
  symbol_t *sym;

  sym = (symbol_t *) malloc (sizeof (symbol_t));
  if (sym == NULL) {
    return (NULL);
  }

  sym->next = NULL;
  sym->name = NULL;
  sym->val = val;
  sym->pass = 0;

  if (name != NULL) {
    sym->name = (char *) malloc (strlen (name) + 1);
    if (sym->name == NULL) {
      free (sym);
      return (NULL);
    }

    strcpy (sym->name, name);
  }

  return (sym);
}

void sym_del (symbol_t *sym)
{
  if (sym != NULL) {
    if (sym->name != NULL) {
      free (sym->name);
    }

    free (sym);
  }
}

void sym_set_val (symbol_t *sym, unsigned val)
{
  sym->val = val;
}

unsigned sym_get_val (symbol_t *sym)
{
  return (sym->val);
}

void sym_set_name (symbol_t *sym, char *name)
{
  if (sym->name != NULL) {
    free (sym->name);
  }

  sym->name = (char *) malloc (strlen (name) + 1);
  if (sym->name == NULL) {
    return;
  }

  strcpy (sym->name, name);
}

char *sym_get_name (symbol_t *sym)
{
  return (sym->name);
}


symtab_t *st_new (void)
{
  symtab_t *st;

  st = (symtab_t *) malloc (sizeof (symtab_t));
  if (st == NULL) {
    return (NULL);
  }

  st->cnt = 0;
  st->sym = NULL;

  return (st);
}

void st_del (symtab_t *st)
{
  symbol_t *sym;

  if (st == NULL) {
    return;
  }

  while (st->sym != NULL) {
    sym = st->sym;
    st->sym = st->sym->next;
    sym_del (sym);
  }

  free (st);
}

void st_sym_add (symtab_t *st, symbol_t *sym)
{
  if (sym != NULL) {
    sym->next = st->sym;
    st->sym = sym;
  }
}

void st_sym_rm (symtab_t *st, symbol_t *sym)
{
  symbol_t *sym1, *sym2;

  if ((st->sym == NULL) || (sym == NULL)) {
    return;
  }

  if (st->sym == sym) {
    st->sym = st->sym->next;
    sym->next = NULL;
    return;
  }

  sym1 = st->sym;
  sym2 = sym1->next;

  while (sym2 != NULL) {
    if (sym2 == sym) {
      sym1->next = sym2->next;
      sym->next = NULL;
      return;
    }

    sym1 = sym2;
    sym2 = sym2->next;
  }
}

symbol_t *st_sym_get (symtab_t *st, char *name)
{
  symbol_t *sym;

  sym = st->sym;

  while (sym != NULL) {
    if (strcmp (sym->name, name) == 0) {
      return (sym);
    }

    sym = sym->next;
  }

  return (NULL);
}

int st_dump (symtab_t *st, FILE *fp)
{
  unsigned n;
  symbol_t *sym;

  sym = st->sym;

  n = 0;

  while (sym != NULL) {
    fprintf (fp, "%u: %s = %u\n", n, sym->name, sym->val);
    n++;

    sym = sym->next;
  }

  fprintf (fp, "%u symbols\n", n);

  return (0);
}

int st_dump_file (symtab_t *st, char *fname)
{
  int  r;
  FILE *fp;

  fp = fopen (fname, "wb");
  if (fp == NULL) {
    return (1);
  }

  r = st_dump (st, fp);

  fclose (fp);

  return (r);
}
