/**************************************************************************
 * HASM64                                                                 *
 **************************************************************************/

/**************************************************************************
 * File name:     symtab.h                                                *
 * Created:       1997-09-04 by Hampa Hug <hhug@student.ethz.ch>          *
 * Last modified: 2001-05-31 by Hampa Hug <hhug@student.ethz.ch>          *
 * Copyright:     (C) 1997-2001 by Hampa Hug <hhug@student.ethz.ch>       *
 **************************************************************************/

/**************************************************************************
 * This program is free software; you can redistribute it and/or modify   *
 * it under the terms of the GNU General Public License as published by   *
 * the Free Software Foundation; either version 2 of the License, or      *
 * (at your option) any later version.                                    *
 **************************************************************************/

#ifndef HASM64_SYMTAB_H
#define HASM64_SYMTAB_H


/*!************************************************************************
 * The symbol type
 **************************************************************************/
typedef struct s_symbol_t {
  struct s_symbol_t *next;
  char              *name;
  unsigned          val;
  unsigned          pass;
} symbol_t;


/*!************************************************************************
 * The symbol table type
 **************************************************************************/
typedef struct {
  unsigned cnt;
  symbol_t *sym;
} symtab_t;


/*!************************************************************************
 * Create a new symbol with name and value.
 **************************************************************************/
symbol_t *sym_new (char *name, unsigned val);

/*!************************************************************************
 * Delete a symbol.
 **************************************************************************/
void sym_del (symbol_t *sym);

/*!************************************************************************
 * Set the symbol value.
 **************************************************************************/
void sym_set_val (symbol_t *sym, unsigned val);

/*!************************************************************************
 * Get the symbol value.
 **************************************************************************/
unsigned sym_get_val (symbol_t *sym);

/*!************************************************************************
 * Set the symbol name. The string is copied.
 **************************************************************************/
void sym_set_name (symbol_t *sym, char *name);

/*!************************************************************************
 * Get the symbol name. The string must not be changed.
 **************************************************************************/
char *sym_get_name (symbol_t *sym);


/*!************************************************************************
 * Create a new empty symbol table.
 **************************************************************************/
symtab_t *st_new (void);

/*!************************************************************************
 * Delete a symbol table and all the symbols in it.
 **************************************************************************/
void st_del (symtab_t *st);

/*!************************************************************************
 * Add a symbol to the symbol table.
 **************************************************************************/
void st_sym_add (symtab_t *st, symbol_t *sym);

/*!************************************************************************
 * Remove a symbol from the symbol table.
 **************************************************************************/
void st_sym_rm (symtab_t *st, symbol_t *sym);

/*!************************************************************************
 * Find a symbol in the symbol table and return a pointer to it. Returns
 * NULL if the symbol is not in the table.
 **************************************************************************/
symbol_t *st_sym_get (symtab_t *st, char *name);

int st_dump (symtab_t *st, FILE *fp);
int st_dump_file (symtab_t *st, char *fname);


#endif
